// SimpleEditorView.cpp : implementation of the CSimpleEditorView class
//

#include "stdafx.h"
#include "SimpleEditor.h"

#include "SimpleEditorDoc.h"
#include "SimpleEditorView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSimpleEditorView

IMPLEMENT_DYNCREATE(CSimpleEditorView, CBCGPEditView)

BEGIN_MESSAGE_MAP(CSimpleEditorView, CBCGPEditView)
	//{{AFX_MSG_MAP(CSimpleEditorView)
	ON_WM_CREATE()
	ON_WM_CONTEXTMENU()
	ON_COMMAND(ID_EDIT_TOGGLEBOOKMARK, OnEditTogglebookmark)
	ON_COMMAND(ID_EDIT_NEXTBOOKMARK, OnEditNextbookmark)
	ON_COMMAND(ID_EDIT_PREVIOUSBOOKMARK, OnEditPreviousbookmark)
	ON_COMMAND(ID_EDIT_FINDPREV, OnEditFindprev)
	ON_COMMAND(ID_EDIT_FINDNEXT, OnEditFindnext)
	//}}AFX_MSG_MAP
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_CURSORPOS, OnUpdateCursorPos)
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CBCGPEditView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CBCGPEditView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CBCGPEditView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSimpleEditorView construction/destruction

CSimpleEditorView::CSimpleEditorView()
{
	// TODO: add construction code here

}

CSimpleEditorView::~CSimpleEditorView()
{
}

BOOL CSimpleEditorView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CBCGPEditView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CSimpleEditorView drawing

void CSimpleEditorView::OnDraw(CDC* pDC)
{
	CSimpleEditorDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	// TODO: add draw code for native data here
}

/////////////////////////////////////////////////////////////////////////////
// CSimpleEditorView printing

BOOL CSimpleEditorView::OnPreparePrinting(CPrintInfo* pInfo)
{
	return CBCGPEditView::OnPreparePrinting(pInfo);
}

void CSimpleEditorView::OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo)
{
	CBCGPEditView::OnBeginPrinting(pDC, pInfo);
	// TODO: add extra initialization before printing
}

void CSimpleEditorView::OnEndPrinting(CDC* pDC, CPrintInfo* pInfo)
{
	// TODO: add cleanup after printing
	CBCGPEditView::OnEndPrinting(pDC, pInfo);
}

/////////////////////////////////////////////////////////////////////////////
// CSimpleEditorView diagnostics

#ifdef _DEBUG
void CSimpleEditorView::AssertValid() const
{
	CBCGPEditView::AssertValid();
}

void CSimpleEditorView::Dump(CDumpContext& dc) const
{
	CBCGPEditView::Dump(dc);
}

CSimpleEditorDoc* CSimpleEditorView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CSimpleEditorDoc)));
	return (CSimpleEditorDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CSimpleEditorView message handlers

int CSimpleEditorView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CBCGPEditView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// Setting fonts 
	ResetDefaultFont();

	return 0;
}

void CSimpleEditorView::OnInitialUpdate() 
{
	CBCGPEditView::OnInitialUpdate();
	
	ASSERT_VALID (GetEditCtrl ());

	// Load XML settings for syntax highlighting
	const CString strXMLFileName = _T("bcgpedit-cpp.xml");
	GetEditCtrl ()->LoadXMLSettings(strXMLFileName);

	// Enable IntelliSense support 
	GetEditCtrl ()->EnableIntelliSense(FALSE);

	// Preset Markers
	const CString& strPathName = GetDocument()->GetPathName();
	GetEditCtrl ()->DeleteAllMarkers ();
	if (strPathName.IsEmpty())
	{
		GetEditCtrl ()->ToggleMarker (3, g_dwBCGPEdit_BookMark);
		GetEditCtrl ()->ToggleMarker (11, g_dwBCGPEdit_BookMark);
		GetEditCtrl ()->ToggleMarker (20, g_dwBCGPEdit_BookMark);
	}

	if (strPathName.IsEmpty())
	{
		RestoreDefaultData();
	}
}

void CSimpleEditorView::ResetDefaultFont()
{
	ASSERT_VALID (GetEditCtrl ());
	
	HFONT hFont = (HFONT)::GetStockObject(DEFAULT_GUI_FONT);
	CDC* pDC = GetDC();

	CFont* pFont = pDC->SelectObject(CFont::FromHandle(hFont));
	pDC->SelectObject(pFont);
	::DeleteObject(hFont);

	LOGFONT lf;
	pFont->GetLogFont (&lf);

	CString strFontName(_T("Courier New"));
	CopyMemory(lf.lfFaceName,(LPCTSTR)strFontName,(strFontName.GetLength() + 1) * sizeof(TCHAR));
	lf.lfWidth = 0;
	lf.lfEscapement = 0;
	lf.lfOrientation = 0;
	lf.lfWeight = FW_NORMAL;
	lf.lfItalic = 0;
	lf.lfUnderline = 0;
	lf.lfStrikeOut = 0;

	m_Font.CreateFontIndirect(&lf);
	GetEditCtrl ()->SetFont (&m_Font);
}

BOOL CSimpleEditorView::RestoreDefaultData()
{
	ASSERT_VALID (GetEditCtrl ());

	// Load CPP example
	BOOL bRet = FALSE;
	LPCTSTR lpszResourceName = MAKEINTRESOURCE (ID_EXAMPLE_CPP);
	ASSERT(lpszResourceName != NULL);

	HINSTANCE hInst = AfxFindResourceHandle(lpszResourceName, _T("Examples"));
	HRSRC hRsrc = ::FindResource(hInst, lpszResourceName, _T("Examples"));
	
	if (hRsrc == NULL)
	{
		ASSERT(FALSE);
	}
	else
	{
		HGLOBAL hGlobal = LoadResource(hInst, hRsrc);

		if (hGlobal == NULL)
		{
			ASSERT(FALSE);
		}
		else
		{
			LPSTR lpszText = (LPSTR)LockResource(hGlobal);

			if (lpszText == NULL)
			{
				ASSERT(FALSE);
			}
			else
			{
				USES_CONVERSION;
				GetEditCtrl ()->SetWindowText(A2T(lpszText));
				bRet = TRUE;
			}

			UnlockResource(hGlobal);
			FreeResource(hGlobal);
		}
	}

	return bRet;
}

void CSimpleEditorView::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	ASSERT_VALID (GetEditCtrl ());

	CPoint pt(point);

	ScreenToClient(&pt);
	if (pt.x < GetEditCtrl ()->m_nLeftMarginWidth)
	{
		pt.x = GetEditCtrl ()->m_nLeftMarginWidth;
		GetEditCtrl ()->SetCaret(GetEditCtrl ()->HitTest(pt));

		CMenu menu;
		if (menu.LoadMenu(IDR_POPUP_SELECTIONMARGIN))
		{
			CMenu* pPopup = menu.GetSubMenu(0);
			ASSERT(pPopup != NULL);

			pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y, AfxGetMainWnd());
		}
	}
	else
	{
		CMenu menu;
		if (menu.LoadMenu(IDR_CONTEXT_MENU))
		{
			CMenu* pPopup = menu.GetSubMenu(0);
			ASSERT(pPopup != NULL);

			pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y, AfxGetMainWnd());
		}
	}
}

void CSimpleEditorView::OnEditTogglebookmark() 
{
	ASSERT_VALID (GetEditCtrl ());
	GetEditCtrl ()->ToggleMarker (GetEditCtrl ()->GetCurRow (), g_dwBCGPEdit_BookMark);
}

void CSimpleEditorView::OnEditNextbookmark() 
{
	ASSERT_VALID (GetEditCtrl ());
	GetEditCtrl ()->GoToNextMarker(g_dwBCGPEdit_BookMark, TRUE);
}

void CSimpleEditorView::OnEditPreviousbookmark() 
{
	ASSERT_VALID (GetEditCtrl ());
	GetEditCtrl ()->GoToNextMarker(g_dwBCGPEdit_BookMark,FALSE);
}

void CSimpleEditorView::OnEditFindnext() 
{
	m_dwFindMask |= FR_DOWN;
	OnFindReplace(0, 0);
}

void CSimpleEditorView::OnEditFindprev() 
{
	m_dwFindMask &= ~FR_DOWN;
	OnFindReplace(0, 0);
}

BOOL CSimpleEditorView::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo) 
{
	ASSERT_VALID (GetEditCtrl ());
	GetDocument()->SetModifiedFlag(GetEditCtrl ()->IsModified());

	return CBCGPEditView::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}

void CSimpleEditorView::OnUpdateCursorPos(CCmdUI *pCmdUI)
{
	ASSERT_VALID (GetEditCtrl ());

	CString str;
	str.Format(_T("Ln %i, Col %i"), GetEditCtrl ()->GetCurRow ()+1, GetEditCtrl ()->GetCurColumn ()+1);
	pCmdUI->SetText (str);

	pCmdUI->Enable (); 
}
